/*
  Function : initialization of SPI
  Arguments : none
  Returns : none
*/
void SPI_Init(void);

/*
  Function : sends/receives data to/from spi device
  Arguments : 
    data - 8bit data to be sent
  Returns : 8bit data received from spi device
*/
unsigned char SPI_TXRX(unsigned char data);

#define    AT1M   0x3
#define    AT2M   0x5
#define    AT4M   0x7
#define    AT8M   0x9
#define    AT16M  0xB
#define    AT32M  0xD
#define    AT64M  0xF
#define    AT128M 0x4
#define    AT256M 0x6 
#define    AT512M 0x8 

/*
  Function : wait for CS low to high transition
  Arguments : none
  Returns : none
*/
void AT45_Reset(void);

/*
  Function : waits until memory is ready
  Arguments : none
  Returns : none
*/
void AT45_Ready(void);

/*
  Function : detects type of dataflash
  Arguments : none
  Returns :
    Hex   Binary  Device Density
    0x3   0 0 1 1 1M
    0x5   0 1 0 1 2M
    0x7   0 1 1 1 4M
    0x9   1 0 0 1 8M
    0xB   1 0 1 1 16M
    0xD   1 1 0 1 32M
    0xF   1 1 1 1 64M
    0x4   0 1 0 0 128M
    0x6   0 1 1 0 256M
    0x8   1 0 0 0 512M
*/
unsigned char AT45_DetectDevice(void);

/*
  Function : reads dataflash memory
  Arguments : 
    address 32bit (r,PA12-PA0,BA9-BA0)
    r - reserved
    PA12-PA0 - specify page to be read
    BA9-BA0 - specify the starting byte address of page

    pointer_ram - pointer to array to be written with data 
    n - number of bytes to be read
  Returns : none
*/
void AT45_ContinousArrayRead(unsigned long, unsigned char * ,unsigned long);
/*
  Function : programs main memory
  Arguments : 
    buffer - has to be '1' or '2'
    address 32bit (r,PA12-PA0,BA9-BA0)
      r - reserved
      PA12-PA0 - specify page to be written
      BFA9-BFA0 - select the first byte in page to be written
    pointer_ram - pointer to array with data to be written
    n - number of bytes to write
  Returns : none
*/
void AT45_MainMemoryProgram(unsigned char , unsigned long , unsigned char *  ,unsigned long );

void AT45_PageErase(unsigned long);
